// cinnamon - the Open Enterprise CMS project
// Copyright (C) 2007-2009 Horner GmbH (http://www.horner-project.eu)
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
// (or visit: http://www.gnu.org/licenses/lgpl.html)

package server;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.hibernate.annotations.Type;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import server.dao.*;
import server.data.ObjectSystemData;
import server.exceptions.CinnamonException;
import server.global.Constants;
import server.index.Indexable;
import server.interfaces.XmlConvertable;
import utils.HibernateSession;
import utils.ParamParser;

import javax.persistence.*;
import java.io.Serializable;
import java.util.*;

@Entity
@Table(name = "folders",
		uniqueConstraints = {@UniqueConstraint(columnNames={"name", "parent_id"})}
)
public class Folder
	implements Serializable, Ownable, Indexable, XmlConvertable {
	
	private static final long	serialVersionUID	= 1L;
	static final DAOFactory daoFactory = DAOFactory.instance(DAOFactory.HIBERNATE);
	
	@Transient
	private transient Logger log = LoggerFactory.getLogger(this.getClass());
	
	@Id @GeneratedValue //(strategy=GenerationType.SEQUENCE)
	@Column(name = "id")
	private long id;

	@Column(name = "name",
			length = Constants.NAME_LENGTH,
			nullable = false)
	private String name;
		
	@Column(name = "metadata",
			length = Constants.METADATA_SIZE,
			nullable = false
			)
	@Type(type="text")
	private String metadata = "<meta/>";

	@ManyToOne
	@JoinColumn(name = "owner_id",
	    nullable = false)
    private User owner;
	
	@ManyToOne
	@JoinColumn(name = "parent_id",
			nullable = true)
	private Folder parent;

	@Column(name = "index_ok",
			nullable = true)
	private Boolean indexOk = true; // a new folder should always be valid and indexed.
	
	@OneToMany(mappedBy = "parent")
	@OrderBy("name")
	private Set<Folder> children = new LinkedHashSet<Folder>();
	
	@ManyToOne
	@JoinColumn(name = "type_id",
			nullable = false)
	private FolderType type;
	
	@ManyToOne
	@JoinColumn(name = "acl_id",
			nullable = false)
	private Acl acl;
	
	@Temporal(TemporalType.TIMESTAMP)
	@Column(name = "indexed",
			nullable = true)
	private Date indexed = new Date();
	/*
	 * Make sure you _do_ index the object!
	 */
	
	@Version
	@Column(name="obj_version")
	@SuppressWarnings("unused")
	private Long obj_version = 0L;
	
	public Folder(){	

	}

	public Folder(String name, String metadata, Acl acl, Folder parent, User owner, FolderType type){
		this.name = name;
		setMetadata(metadata);
		this.acl = acl;
		this.parent = parent;
		this.owner = owner;
		this.type = type;
	}
	
	public Folder(Hashtable<String, String> cmd){
		EntityManager em = HibernateSession.getLocalEntityManager();
		// TODO: change constructor to receive valid EM	
		FolderDAO folderDao = daoFactory.getFolderDAO(em);
		
		log.debug("Entering Folder-Constructor");
				
		this.name = cmd.get("name");
		this.setMetadata(cmd.get("metadata"));
		Long parentId= ParamParser.parseLong(cmd.get("parentid"), "error.parse.parent_id");
		
		log.debug("trying to find parent folder with id "+parentId);

		
		if(parentId == 0){			
			this.parent = folderDao.findRootFolder();
			log.debug("got root folder:  "+parent);			
		}
		else{
			Folder myParent = folderDao.get(parentId); 
			if(myParent == null ){ // parent not found
				throw new CinnamonException("error.parent_folder.not_found");
			}
			else{
				this.parent = myParent;
			}
		}

		// typeid is optional (at least for old clients).
		if(! cmd.containsKey("typeid")){
			cmd.put("typeid", "0");
			log.warn("old client did not submit type_id.");
		}
		Long folderTypeId = ParamParser.parseLong(cmd.get("typeid"), "error.param.type_id");
		FolderTypeDAO ftDao = daoFactory.getFolderTypeDAO(em);
		if(folderTypeId == 0){
			this.type = ftDao.findByName(Constants.FOLDER_TYPE_DEFAULT);
		}
		else{
			this.type = ftDao.get(folderTypeId);
		}
		
		log.debug("check if folder already exists.");
		
    	// check whether folder exists in parent folder
	    List<Folder> folders = folderDao.findAllByNameAndParentID(name, parent);
    	if(! folders.isEmpty()){
    		throw new CinnamonException("error.folder.exists");
    	}
  	
    	log.debug("set acl for this folder");
    	// Set ACL for this folder:
    	if(cmd.containsKey("aclid")){
    		Long acl_id = ParamParser.parseLong(cmd.get("aclid"), "error.parse.acl_id");
    		AclDAO aclDao = daoFactory.getAclDAO(em);		
    		this.acl = aclDao.get(acl_id);
    	}
    	else{
    		// a folder normally inherits the parent folder's acl.
    		this.acl = parent.acl;
    	}
    	
		Long ownerId = ParamParser.parseLong(cmd.get("ownerid"), "error.param.owner_id");
		UserDAO userDao = daoFactory.getUserDAO(em);
		User myOwner = userDao.get(ownerId);
		if(myOwner == null){
			throw new CinnamonException("error.user.not_found");
		}
		owner = myOwner;
	}

    public Folder(Hashtable<String,String> cmd, User owner){
    	this(cmd);
    	this.owner = owner;	
    }
    

	public long getId() {
		return id;
	}

	public String getMetadata() {		
		return metadata;
	}

	/**
	 * Set the Metadata on this folder. Tries to parse the submitted string
	 * and throws an exception if it is not valid XML.
	 * @param metadata
	 * 
	 */
	public void setMetadata(String metadata) {
		if(metadata == null || metadata.trim().length() == 0){
			this.metadata = "<meta/>";
		}
		else{			
			ParamParser.parseXmlToDocument(metadata, "error.param.metadata");
			this.metadata = metadata;
		}
	}

	public Folder getParent() {
		return parent;
	}

	public void setParent(Folder parent) {
		this.parent = parent;
	}

	public void setParent(Long parentId){
		// TODO: change setParent to receive valid EM
		EntityManager em = HibernateSession.getLocalEntityManager();
		log.debug("trying to find parent folder with id "+parentId);
		
		if (parentId == 0) {
		    FolderDAO folderDao = daoFactory.getFolderDAO(em);
			parent = folderDao.findRootFolder();
		}
		else{		
			// TODO: use DAO
			Folder myParent = em.find(Folder.class, parentId);		
			if(myParent == null) 
				throw new CinnamonException("error.parent_folder.not_found");
			parent = myParent;
		}
	}
	
	public void setAcl(Long aclId){
		// TODO: change setAcl to receive valid EM
		EntityManager em = HibernateSession.getLocalEntityManager();
		AclDAO aclDAO = daoFactory.getAclDAO(em);
		acl = (aclId == 0) ? aclDAO.findByName(Constants.ACL_DEFAULT) : aclDAO.get(aclId);
		
		if (acl == null) {
			throw new CinnamonException("error.acl.not_found");				
		}		
	}
	
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	@SuppressWarnings("unused")
	private void setId(long id) {
		this.id = id;
	}

	public Acl getAcl() {
		return acl;
	}

	public void setAcl(Acl acl) {
		this.acl = acl;
	}

	static public boolean folderExists(long id){
		EntityManager em = HibernateSession.getLocalEntityManager();
		Folder f = em.find(Folder.class, id);
		return f != null;
	}
	
	static public boolean folderExists(Folder folder){
		EntityManager em = HibernateSession.getLocalEntityManager();
		
		if (folder == null) {
			throw new RuntimeException("error.folder.not_found");
		}
		long id = folder.getId();
		Folder f = em.find(Folder.class, id);
		
		return f != null;
	}
	
	public boolean is_rootfolder(){
		if(getParent() == null){
			return false;
		}
		
		return getParent().getId() == getId();
	}	
	
	/**
	 * @return Folder	The workflow.Server will store its Objects in this Folder. 
	 * 
	 */
	// TODO: use DAO
	static public Folder findWorkflowFolder(EntityManager emx){
		Logger log = LoggerFactory.getLogger("server.Folder");
		FolderDAO folderDao = daoFactory.getFolderDAO(emx);		
		Folder root = folderDao.findRootFolder();
		log.debug("Root-Folder-Id is: "+root.getId());	
		
		Folder f = null;
		try {
			Query q = emx.createQuery("select f from Folder f where f.name=:name and f.parent=:parent");
			q.setParameter("name", Constants.WORKFLOW_FOLDER_NAME);
			q.setParameter("parent", root);
			f = (Folder) q.getSingleResult();
		} catch (Exception e) {
			log.warn("Could not find workflow-Folder"+e.getMessage());
		}
		return f;
	}
	
	/**
	 * 
	 * @return Folder for workflow objects.
	 */
	static public Folder findWorkflowFolder(){
		EntityManager em = HibernateSession.getLocalEntityManager();
		return findWorkflowFolder(em);
	}

	public void setChildren(Set<Folder> children) {
		this.children = children;
	}

	public Set<Folder> getChildren() {
		return children;
	}

	/**
	 * @return the owner
	 */
	public User getOwner() {
		return owner;
	}

	/**
	 * @param owner the owner to set
	 */
	public void setOwner(User owner) {
		this.owner = owner;
	}
	
	/**
	 * Add the folder as XML Element "folder" to the parameter Element.
	 * @param root
	 */
	public void toXmlElement(Element root){
		Element folder = root.addElement("folder");
		folder.addElement("id").addText(String.valueOf(getId()) );
		folder.addElement("name").addText( getName());
		folder.add(User.asElement("owner", getOwner()));
		folder.addElement("aclId").addText(String.valueOf(getAcl().getId()));
		folder.addElement("typeId").addText(String.valueOf(type.getId()));
		if(folder.getParent() != null){
			folder.addElement("parentId").addText(String.valueOf(getParent().getId()));
		}
		else{
			folder.addElement("parentId");
		}
	}
	
	/**
	 * Create a list of folders right up to one step below the root folder (so the
	 * root folder is not returned).
	 * @return List of parent folders
	 */
	List<Folder> getAncestors(){
	    List<Folder> folderList = new ArrayList<Folder>();
	    Folder folder = this;
		while(folder.getParent() != null && folder.getParent() != folder){
    		folderList.add(folder.getParent());
    		folder = folder.getParent();
    	}
	    return folderList;
	}

	@Override
	public String getContent(String repository) {
		/*
		 * Folder.getContent could return a list of children and objects.
		 * But what would be the purpose of this in a search context?
		 * Is there a use case for "give me all folders which contain foo"
		 * instead of searching for all objects/folders named "foo"?
		 */
		return "<content/>";
	}

	@Override
	public String getSystemMetadata() {
		Document doc = DocumentHelper.createDocument();
		Element root = doc.addElement("sysMeta");
		String className = getClass().getName();
		root.addAttribute("javaClass", className);
		root.addAttribute("hibernateId", String.valueOf(getId()));
		root.addAttribute("id", className + "@" + getId());		
		toXmlElement(root);
		return doc.asXML();
	}
	
	/**
	 * Turn a collection of folders into an XML document. Any exceptions encountered during 
	 * serialization are turned into error-Elements which contain the exception's message.
	 * @param results
	 * @return Document
	 */
	static public Document generateQueryFolderResultDocument(Collection<Folder> results) 
		{
		Document doc = DocumentHelper.createDocument();
		Element root = doc.addElement("folders");
		Logger log = LoggerFactory.getLogger(ObjectSystemData.class);
		
		for (Folder	folder : results) {
    		Long id= folder.getId();
    		log.debug("working on object: "+id);
    		try {
    			folder.toXmlElement(root);
    		}
    		catch(CinnamonException ex) {
    			log.error("Error serializing folder: " + id + " - " + ex.getMessage());
    			Element error = DocumentHelper.createElement("error").addText(ex.getLocalizedMessage());
    			error.addElement("id").addText(id.toString());
    			root.add(error);
    		}
    	}
		return doc;
	}

	/**
	 * @return the indexed
	 */
	public Date getIndexed() {
		return indexed;
	}

	/**
	 * @param indexed the indexed to set
	 */
	public void setIndexed(Date indexed) {
		this.indexed = indexed;
	}
	
	/**
	 * 
	 * @param recursive set to true if subfolders must be included in the list of OSDs.
	 * @return a List of OSDs in this folder
	 */
	public List<ObjectSystemData> getOSDList(Boolean recursive){
		EntityManager em = HibernateSession.getLocalEntityManager();
		FolderDAO folderDao = daoFactory.getFolderDAO(em);
		
		List<ObjectSystemData> osds = folderDao.getFolderContent(this, recursive);
		return osds;
	}

	/**
	 * @return the type
	 */
	public FolderType getType() {
		return type;
	}

	/**
	 * @param type the type to set
	 */
	public void setType(FolderType type) {
		this.type = type;
	}

	/**
	 * @return the indexOk
	 */
	public Boolean getIndexOk() {
		return indexOk;
	}

	/**
	 * @param indexOk the indexOk to set
	 */
	public void setIndexOk(Boolean indexOk) {
		this.indexOk = indexOk;
	}
	
	
	@Override
	/*
	 * Implements Comparable interface based on id.
	 * This is used to generate search results with a consistent ordering (where the
	 * results are not ordered by any other parameter).
	 * 
	 */
	public int compareTo(XmlConvertable o) {
		if(getId() > o.getId()){
			return 1;
		}
		else if(getId() < o.getId()){
			return -1;
		}
		return 0;
	}
}
