// cinnamon - the Open Enterprise CMS project
// Copyright (C) 2007-2009 Horner GmbH (http://www.horner-project.eu)
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
// (or visit: http://www.gnu.org/licenses/lgpl.html)

package server;

import org.dom4j.Element;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import server.dao.DAOFactory;
import server.dao.PermissionDAO;
import server.exceptions.CinnamonException;
import server.global.Constants;
import server.i18n.LocalMessage;
import utils.HibernateSession;

import javax.persistence.*;
import java.io.Serializable;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Set;

@Entity
@Table(name = "permissions",
		uniqueConstraints = {@UniqueConstraint(columnNames={"name"})}		
)
public class Permission implements Serializable{
	
	/**
	 * 
	 */
	private static final long	serialVersionUID	= 1L;
	static final DAOFactory daoFactory = DAOFactory.instance(DAOFactory.HIBERNATE);
	
//	private transient Logger log = LoggerFactory.getLogger(this.getClass());
	
	@Id @GeneratedValue
	@Column(name = "id")
	private long id = 0;

	@Column(name = "name",
			length = Constants.NAME_LENGTH,
			nullable = false)
	private String name;
	
	@Column(name = "description",
			length = Constants.DESCRIPTION_SIZE,
			nullable = false)
	private String description;
	
	@Version
	@Column(name="obj_version")
	@SuppressWarnings("unused")
	private Long obj_version = 0L;

	@OneToMany(mappedBy = "permission",
			cascade = { CascadeType.PERSIST, CascadeType.REMOVE }
	)
	private Set<AclEntryPermission> aePermissions = new HashSet<AclEntryPermission>();

	
	// Object related permissions:
	/**
	 * Permission to lock an object. Implies permission to unlock the user's own locks.
	 */
	public static final String LOCK							= "_lock";
	
	/**
	 * Permission to write to an object, that is: change its content.
	 */
	public static final String WRITE_OBJECT_CONTENT			= "_write_object_content";
	
	/**
	 * Permission to read to an object's content.
	 */
	public static final String READ_OBJECT_CONTENT			= "_read_object_content";
	
	/**
	 * Permission to change an object's system metadata
	 */
	public static final String WRITE_OBJECT_SYS_METADATA	= "_write_object_sysmeta";
	
	/**
	 * Permission to read an object's system metadata
	 */
	public static final String READ_OBJECT_SYS_METADATA		= "_read_object_sysmeta";
	
	
	/**
	 * Permission to change an object's custom metadata
	 */
	public static final String WRITE_OBJECT_CUSTOM_METADATA = "_write_object_custom_metadata";
	
	/**
	 * Permission to read an object's custom metadata
	 */
	public static final String READ_OBJECT_CUSTOM_METADATA 	= "_read_object_custom_metadata";
	
	/**
	 * Permission to create a new version of an object.
	 */
	public static final String VERSION_OBJECT	= "_version";
	
	/**
	 * Permission to delete an object.
	 */
	public static final String DELETE_OBJECT	= "_delete_object";
	
	/**
	 * Permission to browse an object, that is: the server will list it when
	 * queried for the content of a folder or when a search for turns up this
	 * object. 
	 */
	public static final String BROWSE_OBJECT	= "_browse";
	
	// -------------- Folder related permissions: ----------------------------
	/**
	 * Permission to create a folder inside the current one.
	 */
	public static final String CREATE_FOLDER	= "_create_folder";
	
	/**
	 * Permission to delete a folder.
	 */
	public static final String DELETE_FOLDER	= "_delete_folder";
	
	/**
	 * Permission to create an object inside a folder.
	 */
	public static final String CREATE_OBJECT	= "_create_inside_folder"; // create_object_inside_folder
	
	/**
	 * Permission to edit a folder (change name, change metadata).
	 */
	public static final String EDIT_FOLDER		= "_edit_folder";
	
	/**
	 * Permission to browse this folder, that is: the folder will be
	 * displayed in a list of its parent's content or may turn up
	 * during a search.
	 */
	public static final String BROWSE_FOLDER	= "_browse_folder";
	
	// permission affecting both folder and objects:
	/**
	 * Permission to move an object or folder, depending on which it is set.
	 */
	public static final String MOVE	 			= "_move";
	
	/**
	 * Permission to change an object's or folder's ACL.
	 */
	public static final String SET_ACL			= "_set_acl";
	
	// other permissions:
	/**
	 * Permission to execute queries against custom tables. 
	 */
	public static final String QUERY_CUSTOM_TABLE = "_query_custom_table";

	/**
	 * Permission to execute customized queries against the OSD-table. 
	 */
	public static final String EXECUTE_QUERY = "_execute_query";
	
	/**
	 * Permission to create an instance of a template object of some kind.
	 * Current use case is for workflow templates. If you have a create_instance
	 * permission on a workflow template, you may call create workflow on it.
	 */
	public static final String CREATE_INSTANCE = "_create_instance";
	
	public static final String[] defaultPermissions = {
			// object:
			WRITE_OBJECT_CONTENT, READ_OBJECT_CONTENT,
			WRITE_OBJECT_CUSTOM_METADATA, READ_OBJECT_CUSTOM_METADATA, 
			WRITE_OBJECT_SYS_METADATA, READ_OBJECT_SYS_METADATA,
			VERSION_OBJECT, DELETE_OBJECT,
			BROWSE_OBJECT, LOCK,
			
			// folder:
			CREATE_FOLDER, DELETE_FOLDER, BROWSE_FOLDER,
			EDIT_FOLDER, CREATE_OBJECT,
			
			// misc:
			MOVE, SET_ACL,
			QUERY_CUSTOM_TABLE,
			CREATE_INSTANCE,
	};
	
	
	public Permission (){	

	}

	public Permission(Hashtable<String, String> cmd){
		name = cmd.get("name");
		description = cmd.get("description");
	}

	public Permission(String name, String descripton){
		this.name = name;
		this.description = descripton;
	}
	
	public long getId() {
		return id;
	}
	
	@SuppressWarnings("unused")
	private void setId(long id) {
		this.id = id;
	}
	
	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	
	/**
	 * @return the aePermissions
	 */
	@Transient
	public Set<AclEntryPermission> getAePermissions() {
		return aePermissions;
	}

	/**
	 * @param aePermissions the aePermissions to set
	 */
	public void setAePermissions(Set<AclEntryPermission> aePermissions) {
		this.aePermissions = aePermissions;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (int) (id ^ (id >>> 32));
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Permission other = (Permission) obj;
		return id == other.id;
	}
	
	/**
	 * Given a name, fetch(String name) fetches the Permission from the database.
	 * If the requested object cannot be found, a CinnamonException is thrown
	 * @param name
	 * @return the Permission of the given name.
	 * @throws server.exceptions.CinnamonException if there is no Permission by this name.
	 */
	public static Permission fetch(String name){
		EntityManager em = HibernateSession.getLocalEntityManager();
		Logger log = LoggerFactory.getLogger(Permission.class);
		
		PermissionDAO permDao = daoFactory.getPermissionDAO(em);
		Permission p = null;
		try {
			p = permDao.findByName(name);
		} catch (Exception e) {
			log.debug("Permission not found.",e);
		}
		if(p == null){
			throw new CinnamonException("error.permission.not_found");
		}
		return permDao.findByName(name);
	}
	
	public String toString(){
		return getName();
	}
	
	public void toXmlElement(Element root){
		Element permission = root.addElement("permission");
		permission.addElement("id").addText(String.valueOf(getId()) );
		permission.addElement("name").addText(  LocalMessage.loc(getName()));
		permission.addElement("description").addText( LocalMessage.loc(getDescription()));
	}
	
}
