// cinnamon - the Open Enterprise CMS project
// Copyright (C) 2007-2009 Horner GmbH (http://www.horner-project.eu)
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
// (or visit: http://www.gnu.org/licenses/lgpl.html)

package server;


import org.hibernate.annotations.Type;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import server.dao.DAOFactory;
import server.dao.SessionDAO;
import server.exceptions.CinnamonException;
import server.global.ConfThreadLocal;
import server.i18n.Language;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Date;
import java.util.UUID;

//import org.hibernate.annotations.Cascade;

@Entity
@Table(name = "sessions")
public class Session implements Serializable{

		/**
		 * 
	 	*/
		private static final long	serialVersionUID	= 1L;
	
		@Id @GeneratedValue
		@Column(name = "id")	
		private long 	id;
		
		@Column(name = "ticket",
				length = 255)
		private String 	ticket;
		
		@Column(name = "expires")
		private Date expires = new Date();
		
		@Column(name = "lifetime")
		private long 	lifetime;
		
		@Column(name = "username",
				length = server.global.Constants.NAME_LENGTH)
		private String	username;
		
		@Column(name = "machinename",
				length = server.global.Constants.NAME_LENGTH)
		private String	machinename;
		
		@Column(name = "message",
				length = 65000)
		@Type(type="text")
		private String	message;
		
		@ManyToOne
		@JoinColumn(name = "language_id",
			nullable = true)
		private Language language;
		
		@ManyToOne(targetEntity = User.class)
		@JoinColumn(name = "user_id", nullable=false)
		private User	user;
		
		@Version
		@Column(name="obj_version")
		@SuppressWarnings("unused")
		private Long obj_version = 0L;
		
		public Session (){			
		}
		
		public Session(String repository, User user, String machinename, Language language){
			 ticket=UUID.randomUUID().toString() + "@" + repository;
			 this.user = user;
			 this.machinename = machinename;
			 this.language = language;
			 message = "";
			 username = user.getName(); // while we still have direct SQL queries.
			 expires.setTime(expires.getTime()+86400); // for testing
		}
	
		public long getId() {
			return id;
		}
		
		@SuppressWarnings("unused")
		private void setId(long id) {
			this.id = id;
		}
		
		public long getLifetime() {
			return lifetime;
		}
		public void setLifetime(long lifetime) {
			this.lifetime = lifetime;
		}
		
		public String getMachinename() {
			return machinename;
		}
		public void setMachinename(String machinename) {
			this.machinename = machinename;
		}
		
		public String getMessage() {
			return message;
		}
		public void setMessage(String message) {
			this.message = message;
		}
		
		public String getTicket() {
			return ticket;
		}
		public void setTicket(String ticket) {
			this.ticket = ticket;
		}

		public String getUsername() {
			return username;
		}
		
		public void setUsername(String username) {
			this.username = username;
		}

		public Date getExpires() {
			return expires;
		}

		public void setExpires(Date expires) {
			this.expires = expires;
		}

		public User getUser() {
			return user;
		}

		public void setUser(User user) {
			this.user = user;
		}

		/**
		 * @return the language
		 */
		public Language getLanguage() {
			return language;
		}

		/**
		 * @param language the language to set
		 */
		public void setLanguage(Language language) {
			this.language = language;
		}
	
	public void checkForExpiration(){
		if(getExpires().getTime() < new Date().getTime()){
			throw new CinnamonException("error.session.expired");
		}
	}
	
	public void renewSession(Long expirationTime){
		Long newTime = (new Date()).getTime() + expirationTime;
		setExpires( new Date(newTime));
	}
	
	static DAOFactory daoFactory = DAOFactory.instance(DAOFactory.HIBERNATE);
	public static Session initSession(EntityManager em, String ticket, String repositoryName){
		Logger log = LoggerFactory.getLogger(Session.class);
		log.debug("Fetching session-Object with em: "+em);
		
		SessionDAO sessionDAO = daoFactory.getSessionDAO(em);
		Session session = sessionDAO.findByTicket(ticket);
		log.debug("retrieved result: "+session);
		User user 	= session.getUser();					
			
		boolean validUser = user != null;
		if(!validUser){
			log.debug("Invalid session ticket");
			throw new CinnamonException("error.session.invalid");
		}			
		log.debug("Got session-Object: "+session+ " for User "+user.getName());
			
		// prevent a user from using an expired session:
		session.checkForExpiration();
		Long sessionExpirationTime =
			ConfThreadLocal.getConf().getSessionExpirationTime(repositoryName);
		log.debug("Session is valid");
		session.renewSession(sessionExpirationTime);
		log.debug("Session renewed until "+session.getExpires());
		return session;
	}
}
