// cinnamon - the Open Enterprise CMS project
// Copyright (C) 2007 Dr.-Ing. Boris Horner
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

package server.dao;

import server.Folder;
import server.data.ObjectSystemData;
import server.exceptions.CinnamonException;

import java.util.List;
import java.util.Map;

/**
 * The DAO interface for a particular entity extends the generic interface and provides the type arguments.
 * 
 * see <a href="http://www.hibernate.org/328.html">Generic Data Access Objects</a>
 * @author Stefan Rother
 *
 */
public interface FolderDAO extends GenericDAO<Folder, Long> {
//	/**
//	 * Returns the folder with the given id.
//	 * @param id
//	 * @return Folder data in the form ~~id/name~~, one line per folder below root.
//	 */
//	List<String> findAllByID(Long id);	// "getFolderById"
//	// TODO: refactor to remove findAllByID
	
	/**
	 * Returns all folders for the given parent id.
	 * @param parentId
	 */
	List<Folder> findAllByParentID(Long parentId);

	/**
	 * Returns the folder with the given path.
	 * @param path path in the form/folder1/folder2/...
	 * @return List of Folders in path (except root folder)
	 * @throws server.exceptions.CinnamonException on error
	 */
	List<Folder> findAllByPath(String path);	// "getFolderByPath"

	/**
	 * Returns the subfolders of the folder with the given id.
	 * @param parentId parent folder id
	 * @return List of folders
	 */
	List<Folder> getSubfolders(Long parentId);

	/**
	 * Delete an empty folder specified by the id-parameter.
	 * @param id folder id
	 */
	void delete(Long id);

	/**
	 * Set name and or description of an folder.
	 * currently accepted fields
	 * <ul> 
	 * 		<li>[parentid] = new parent id</li>
	 * 		<li>[name]= new name</li>
	 * </ul>
	 * 
	 * @param id	the id of the folder (integer)
	 * @param fields map of the fields to be set
	 * @throws server.exceptions.CinnamonException if the parent folder doesn't exist
	 */
	Folder update(Long id, Map<String, String> fields);
	
	/**
	 * @see #update(Long, java.util.Map)
	 * @param folder
	 * @param fields
	 */
	Folder update(Folder folder, Map<String ,String> fields);

	/**
	 * Installation-Hint<br>
	 * Create a Folder whose parent equals it's own id and whose name is equals the ROOT_FOLDER_NAME.
	 * This is the default folder in which objects and folders are created if no parent_id is given.
	 * 
	 * @return	rootFolder
	 */
	Folder findByNameAndParentID(String name, Long id);
	//TODO: is it correct to use ID instead of Id?
	
	List<Folder> findAllByNameAndParentID(String name, Folder parent);
	
	/**
	 * Find all Folders where index_ok is NULL. Those are the ones
	 * whose index is not current.
	 * @param maxResults maximum number of results
	 * @return List of Folders to index (limited by maxResults).
	 */
	List<Folder> findIndexTargets(Integer maxResults);
	
	/**
	 * Set the indexed-column to 0 and trigger a re-indexing by the IndexServer.
	 */
	Integer prepareReIndex();
	
	Folder get(String id);
	
	Folder findRootFolder();
	
	List<ObjectSystemData> getFolderContent(Folder folder, Boolean recursive);
	
	/**
	 * Return all ancestor-Folders 
	 * up to but not including the root folder.
	 */
	List<Folder> getParentFolders(Folder folder);
	
	Folder findByPath(String path);
}
