// cinnamon - the Open Enterprise CMS project
// Copyright (C) 2007 Dr.-Ing. Boris Horner
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

package server.dao;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import server.Folder;
import server.ObjectType;
import server.Relation;
import server.User;
import server.data.ContentStore;
import server.data.ObjectSystemData;
import utils.ParamParser;

import javax.persistence.NoResultException;
import javax.persistence.Query;
import java.util.List;

public class ObjectSystemDataDAOHibernate extends
		GenericHibernateDAO<ObjectSystemData, Long> implements
		ObjectSystemDataDAO {
	private final Logger log = LoggerFactory.getLogger("server.dao.ObjectSystemDataDAOHibernate");
	static final 	DAOFactory factory = DAOFactory.instance(DAOFactory.HIBERNATE);
	
//	@Override
//	public ObjectSystemData findByName(String name) {
//		Query q = getSession().createQuery("select o from ObjectSystemData o where name=:name");
//		q.setParameter("name", name);
//		return (ObjectSystemData) q.getSingleResult();
//	}

	@SuppressWarnings("unchecked")
	@Override
	public List<ObjectSystemData> findAllByParent(Long parentID) {
		Query q = getSession().createQuery(
		"select o from ObjectSystemData o where parent_id=:parentid")
		.setParameter("parentid", parentID);
		return q.getResultList();
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<ObjectSystemData> findAllByPredecessorID(ObjectSystemData predecessor) {
		Query q = getSession().createNamedQuery("findOsdsByPredecessor");
		q.setParameter("predecessor", predecessor);
		return q.getResultList();
	}

	@Override
	public ObjectSystemData get(Long id) {
		return getSession().find(ObjectSystemData.class, id);
	}

	@Override
	public void delete(Long id) {
		log.debug("before loading osd");	
	    ObjectSystemData osd = get(id);
	    if(osd == null) {
    		throw new RuntimeException("error.object.not_found");
    	}
    	log.debug("Found osd");
		ObjectSystemData predecessor = osd.getPredecessor();
		
    	log.debug("checking for descendants ");
		boolean hasDescendants = findAllByPredecessorID(osd).size() > 0;
    	if(hasDescendants) {
    		// DEBUG
//    		log.debug("object "+osd.getId()+" has descendants:");
//    		for(ObjectSystemData o : findAllByPredecessorID(osd)){
//    			log.debug("found:"+o.getId());
//    		}
    		// EoDEBUG
    		throw new RuntimeException("error.delete.has_descendants");
    	}
        
    	// TODO: check for protected relations
    		    	
    	// delete relations
    	Long osd_id = osd.getId();
	    RelationDAO relationDAO = factory.getRelationDAO(getSession());
	    List<Relation> relations = relationDAO.findAllByLeftOrRightID(osd_id, osd_id);
    	for (Relation rel : relations) {
    		relationDAO.makeTransient(rel);
    	}	    	

    	makeTransient(osd);
    	log.debug("object deleted.");
    	
    	/*
    	 * An object is latestBranch, if it has no descendants. 
    	 * You can only delete an object without descendants.
    	 * So, the predecessor's only child has been deleted and this
    	 * makes the predecessor latestBranch.
    	 * 
    	 * An object is latestHead, if it is not of part of a branch and has no
    	 * descendants. As we already said, this predecessor cannot have any
    	 * descendants and so we can set latestHead to true if it is part of
    	 * the main branch (no . in version).
    	 */
    	if(predecessor != null){
    		predecessor.setLatestBranches(true);
    		if(! predecessor.getVersion().contains(".")){
    			predecessor.setLatestHead(true);
    		}
    	}
    	
    	ContentStore.deleteObjectFile(osd);	
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<ObjectSystemData> findAllByRootAndNotSelf(ObjectSystemData osd) {
		Query q = getSession().createNamedQuery("findOSDByRootAndNotSelf");
		q.setParameter("root", osd.getRoot());
		q.setParameter("self", osd);
		return q.getResultList();
	}

	/* (non-Javadoc)
	 * @see server.dao.ObjectSystemDataDAO#list()
	 */
	@SuppressWarnings("unchecked")
	@Override	
	public List<ObjectSystemData> list() {
		Query q = getSession().createNamedQuery("selectAllOSDs");
		return q.getResultList();
	}
	
	@Override
	public ObjectSystemData get(String id){
		Long osdId = ParamParser.parseLong(id, "error.param.id"); 
		return get(osdId);
	}
	
	@SuppressWarnings("unchecked")
	public List<ObjectSystemData> findIndexTargets(Integer maxResults){
		Query q = getSession().createNamedQuery("findObjectIndexTargets");
		q.setMaxResults(maxResults);
		return q.getResultList();
	}

	@Override
	public Integer prepareReIndex() {
		Query q = getSession().createNamedQuery("prepareOSD_ReIndex");
		return q.executeUpdate();
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<ObjectSystemData> findAllVersions(ObjectSystemData osd) {
		if(! osd.getVersion().equals("1")){
			osd = osd.getRoot();
		}
		Query q = getSession().createNamedQuery("findAllVersionsOfOsdOrderByIdDesc");
		q.setParameter("root", osd);
		return q.getResultList();
	}

	@Override
	public ObjectSystemData findByRootAndVersion(ObjectSystemData osd,
			String version) {
		Query q = getSession().createNamedQuery("findOsdByRootAndVersion");
		q.setParameter("root", osd.getRoot());
		q.setParameter("version", version);
		ObjectSystemData result;
		try{
			result = (ObjectSystemData) q.getSingleResult();
		}
		catch (NoResultException e) {
			result = null;
		}
		return result; 
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public List<ObjectSystemData> findAllVersionsOrderLastToFirst(ObjectSystemData osd) {
		Query q = getSession().createNamedQuery("findOSDAllVersionsLastToFirst");
		q.setParameter("root", osd.getRoot());
		return q.getResultList();
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<ObjectSystemData> findAllByTypeAndProcstate(ObjectType type,
			String procstate) {
		Query q = getSession().createNamedQuery("findAllOsdByTypeAndProcstate");
		q.setParameter("type", type);
		q.setParameter("procstate", procstate);
		return q.getResultList();
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<ObjectSystemData> findAllByTypeAndProcstateAndOwner(
			ObjectType type, String procstate, User owner) {
		Query q = getSession().createNamedQuery("findAllOsdByTypeAndProcstateAndOwner");
		q.setParameter("type", type);
		q.setParameter("procstate", procstate);
		q.setParameter("owner", owner);
		return q.getResultList();
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<ObjectSystemData> findAllByTypeAndProcstateAndOwnerAndRelationToLeftOsd(
			ObjectType type, String procstate, User owner,
			ObjectSystemData related) {
		Query q = getSession().createNamedQuery("findAllOsdByTypeAndProcstateAndOwnerAndRelationToLeftOsd");
		q.setParameter("type", type);
		q.setParameter("procstate", procstate);
		q.setParameter("owner", owner);
		q.setParameter("leftOSD", related);
		return q.getResultList();
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<ObjectSystemData> findAllByTypeAndProcstateAndRelationToLeftOsd(
			ObjectType type, String procstate, ObjectSystemData related) {
		Query q = getSession().createNamedQuery("findAllOsdByTypeAndProcstateAndRelationToLeftOsd");
		q.setParameter("type", type);
		q.setParameter("procstate", procstate);
		q.setParameter("leftOSD", related);
		return q.getResultList();
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<ObjectSystemData> findAllByParentAndName(Folder parent,
			String name) {
		Query q = getSession().createNamedQuery("findOsdsByParentAndName");
		q.setParameter("parent", parent);
		q.setParameter("name", name);
		return q.getResultList();
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<ObjectSystemData> findAllByNameAndType(String name, ObjectType type) {
		Query q = getSession().createNamedQuery("findOsdsByNameAndType");
		q.setParameter("name", name);
		q.setParameter("type", type);
		return q.getResultList();
	}
	
}
