// cinnamon - the Open Enterprise CMS project
// Copyright (C) 2007 Dr.-Ing. Boris Horner
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

package server.data;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.hibernate.annotations.Type;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import server.*;
import server.dao.*;
import server.exceptions.CinnamonException;
import server.global.Conf;
import server.global.ConfThreadLocal;
import server.global.Constants;
import server.i18n.Language;
import server.index.Indexable;
import server.interfaces.XmlConvertable;
import server.lifecycle.LifeCycle;
import server.lifecycle.LifeCycleState;
import utils.HibernateSession;
import utils.ParamParser;

import javax.persistence.*;
import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.util.*;

@Entity
@Table(name="objects")
public class ObjectSystemData
	implements Serializable, Ownable, Indexable, XmlConvertable{
	private static final long	serialVersionUID	= 1L;
	static final DAOFactory daoFactory = DAOFactory.instance(DAOFactory.HIBERNATE);
	@Transient
	transient Logger log = LoggerFactory.getLogger(this.getClass());
	
	@Column(name ="contentpath",
			length = 255,// length 255 should be enough, as we use a 128 Bit UUID for the file/folder structure. 
			nullable = true
			) 
	private String contentPath = null;
	
	@Column(name = "contentsize",
			nullable = true)
	private Long contentSize = null;
	
	@Column(name = "name",
			length = Constants.NAME_LENGTH,
			nullable = false)
	private String name="";
	
	@Column(name = "index_ok",
			nullable = true)
	private Boolean indexOk = true; // a new object should always be valid and indexed.
	
	@OneToOne
	@JoinColumn(name = "pre_id",
				nullable = true)
	private ObjectSystemData predecessor;
	
	@ManyToOne
	@JoinColumn(name = "root_id",
				nullable = true)
				// IMPLEMENTATION: workflow-objects set root to null.
				// TODO: fix workflow: root_id should be now NOT NULL.
				// TODO: fix OSD creation.
	private ObjectSystemData root;
	
	@Id @GeneratedValue
	@Column(name = "id")
	private long id =0;
	
	@ManyToOne
	@JoinColumn(name = "creator_id",
				nullable = false)
	private User creator;
	
	@ManyToOne
	@JoinColumn(name = "locked_by",
				nullable = true)
	private User locked_by;
	
	@ManyToOne
	@JoinColumn(name = "modifier_id",
			nullable = false)
	private User modifier;
	
	@ManyToOne
	@JoinColumn(name = "owner_id",
			nullable = true)
	private User owner;
	
	@Temporal(TemporalType.TIMESTAMP)
	@Column(name = "created",
			nullable = false)
	private Date created = new Date();
	
	@Temporal(TemporalType.TIMESTAMP)
	@Column(name = "indexed",
			nullable = true)
	private Date indexed = null;
	
	@Temporal(TemporalType.TIMESTAMP)
	@Column(name = "modified",
			nullable = false)
	private Date modified = new Date();
	
	@ManyToOne
	@JoinColumn(name = "lang_id",
				nullable=false)
	private Language language;
	
	@ManyToOne
	@JoinColumn(name = "acl_id",
			nullable = false)
	private Acl acl;
	
	@ManyToOne
	@JoinColumn(name = "parent_id",
				nullable = false)
	private Folder parent;
	
	//private ObjectSystemData pre_;
	@ManyToOne
	@JoinColumn(name = "format_id",
			nullable = true)
	private Format format;
	
	@ManyToOne
	@JoinColumn(name = "type_id",
				nullable = false)
	private ObjectType type;
	
	@Column(name = "appname",
			length = 255)
	private String appName = "";
	
	@Column(name = "metadata",
			length = Constants.METADATA_SIZE,
			nullable = false
			)
	@Type(type="text")
	private String metadata = "<meta/>";
	
	@Column(name = "procstate",
			length = 128)
	private String procstate;
	
	@Column(name = "latesthead",
			nullable = false)
	private Boolean latestHead;
	
	@Column(name = "latestbranches",
			nullable = false)
	private Boolean latestBranches = true;
	
	@Column(name = "version",
			length = 128,
			nullable = false)
	private String version;
		
	@Version
	@Column(name="obj_version")
	@SuppressWarnings("unused")
	private Long obj_version = 0L;

    @ManyToOne
	@JoinColumn(name = "lifecycle_state_id",
				nullable=true)
	private LifeCycleState state;
	
//	@Transient
//	private transient EntityManager em = HibernateSession.getLocalEntityManager();
	
	public ObjectSystemData() {
	
	}
		
	public ObjectSystemData(Hashtable<String, Object> cmd, User user, boolean initFromPredecessor){
		// TODO: change constructor to expect valid EM
		EntityManager em = HibernateSession.getLocalEntityManager();
		ObjectSystemDataDAO osdDao = daoFactory.getObjectSystemDataDAO(em);
		
		log.debug("predecessor-init");
		if(cmd.containsKey("preid")) {
			Long preId = ParamParser.parseLong((String)cmd.get("preid"), "error.param.pre_id");			
			ObjectSystemData pred = osdDao.get(preId);
			if(pred == null){
				throw new RuntimeException("error.predecessor.not_found");
			}
			
			if(initFromPredecessor) {
				name = pred.getName();
				parent =pred.getParent();
				metadata =pred.getMetadata();
				appName = pred.getAppName();
				language = pred.getLanguage();
			}
			this.predecessor = pred;
			/*
			 * because sys is now the latest obj in the branch,
			 * its predecessor has to loose the latestBranches flag.
			 */
			predecessor.setLatestBranches(false);
			predecessor.setLatestHead(false);
//			log.debug("flushing after setting predecessor.branches");

		}
		
		log.debug("set version label");
		version = createNewVersionLabel();
		log.debug("new version: "+version);
		latestHead = !version.contains(".");
		
		log.debug("set root");
		if(predecessor == null) {
			setRoot(this);
		}
		else {
			setRoot(predecessor.getRoot());
		}
		
		log.debug("set name,appname,metadata");
		if(cmd.containsKey("name")){
			setName((String)cmd.get("name"));
		}
		if(cmd.containsKey("appname")){
			setAppName((String)cmd.get("appname"));
		}
		if(cmd.containsKey("metadata")){
			setMetadata((String)cmd.get("metadata"));
		}

		/*
		 * Set ObjectType:
		 * 1. by objtype_id
		 * 2. by objtype string
		 * 3. by predecessor
		 * 4. default_object_type.
		 */
		log.debug("set objectType");
		ObjectTypeDAO otDao = daoFactory.getObjectTypeDAO(em);
		if(cmd.containsKey("objtype_id")){
			
			Long otId = ParamParser.parseLong((String)cmd.get("objtype_id"), "error.param.objtype_id");
			this.type = otDao.get(otId);
		}
		else if (cmd.containsKey("objtype")){
		    List<ObjectType> list = otDao.findAllByName((String) cmd.get("objtype"));
		    if (list.isEmpty()) {
		    	throw new CinnamonException("error.param.objtype");								
			}
		    else{
		    	this.type = list.get(0);
		    }
		}
		else if (predecessor != null) {
			this.type = predecessor.getType();
		}
		else{
		    this.type = otDao.findByName(Constants.OBJTYPE_DEFAULT);
		}
		
		log.debug("set parentfolder");
		if(cmd.containsKey("parentid")){			
			Long parent_id =  Long.parseLong( (String)cmd.get("parentid"));
			if (parent_id != 0) {
			    FolderDAO folderDAO = daoFactory.getFolderDAO(em);
			    parent = folderDAO.get(parent_id);
				if (parent != null) {
					setParent(parent);
				}
				else{
					throw new RuntimeException("error.parent_folder.not_found");
				}
			
			} else { // parent_id == 0
				FolderDAO folderDao = daoFactory.getFolderDAO(em);
				Folder rootFolder = folderDao.findRootFolder();
				setParent(rootFolder);
			}
		}
		else if(parent == null){
			// note: parent may be set from predecessor.
			throw new CinnamonException("error.parent_folder.not_found");
		}
		
		log.debug("set format");
		FormatDAO formatDao = daoFactory.getFormatDAO(em);
		if(cmd.containsKey("format_id")){
			Long formatId = ParamParser.parseLong((String)cmd.get("format_id"),"error.param.format_id");
			Format format = formatDao.get(formatId);
			setFormat( format );
		}
		if (cmd.containsKey("format")) {
			Format format = formatDao.findByName((String) cmd.get("format"));
			setFormat(format);
		}

		
		log.debug("set acl");
		if(cmd.containsKey("acl_id")){
			AclDAO aclDao = daoFactory.getAclDAO(em);
			Long aclId = ParamParser.parseLong((String)cmd.get("acl_id"),"error.param.acl_id");
			Acl acl = aclDao.get(aclId);
			setAcl( acl );
		}
		else{
			// if no sepcific acl is given, use the parent folder's acl
			log.debug("set acl to parent-folder's acl");
			setAcl(getParent().getAcl());
		}
		

		procstate = Constants.PS_LABEL_CREATED;
		
		/*
		 * Set language to language_id or to 'und' if language is null.
		 */
		log.debug("set language");
		LanguageDAO langDao = daoFactory.getLanguageDAO(em);
		if(cmd.contains("language_id")){
			Long langId = ParamParser.parseLong((String) cmd.get("language_id"), 
					"error.param.language_id");
			Language lang = langDao.get(langId);
			if(lang == null){
				throw new RuntimeException("error.language.not_found");
			}
			setLanguage(lang);
		}
		else if (getLanguage() == null){
			Language lang = langDao.findByIsoCode("und");
			setLanguage(lang);
		}
		
		log.debug("set owner, modifier, creator");
		setOwner(user);
		setModifier(user);
		setCreator(user);
	}
	
	public String getContentPath() {
		return contentPath;
	}
	
	/**
	 * Set the contentPath directly (and <em>NOT</em> the contentSize). Only for internal use. 
	 * @param contentPath
	 */
	private void setContentPath(String contentPath){
		this.contentPath = contentPath;
	}

	/**
	 * Set the contentPath <em>and the contentSize</em>, if the former is a valid String which can
	 * be mapped to a valid File.
	 * @param contentPath
	 */
	public void setContentPath(String contentPath, String repository) {
		this.contentPath = contentPath;
		if(contentPath != null){			
			this.contentSize = contentPath.length()>0?(new File(getFullContentPath(repository))).length():0;
		}
		else{
			this.contentSize = null;
		}
	}

	public Acl getAcl() {
		return acl;
	}

	public void setAcl(Acl acl) {
		this.acl = acl;
	}

	public String getAppName() {
		return appName;
	}

	public void setAppName(String appName) {
		this.appName = appName;
	}

	public Long getContentSize() {
		return contentSize;
	}

	public void setContentSize(Long contentSize) {
		this.contentSize = contentSize;
	}

	public Format getFormat() {
		return format;
	}

	public void setFormat(Format format) {
		this.format = format;
	}

	public long getId() {
		return id;
	}

	@SuppressWarnings(value = { "unused" })
	private void setId(long id) {
		this.id = id;
	}

	public String getMetadata() {
		return metadata;
	}
	
	/**
	 * Set the Metadata on this object. Tries to parse the submitted string
	 * and throws an exception if it is not valid XML.
	 * @param metadata
	 * 
	 */
	public void setMetadata(String metadata ) {
		if(metadata.trim().length() == 0){
			metadata = "<meta/>";
		}
		else{			
			ParamParser.parseXmlToDocument(metadata, "error.param.metadata");
		}
		this.metadata = metadata;
	}

	public void setName(String name) {
		this.name = name;
	}

	public Folder getParent() {
		return parent;
	}

	public void setParent(Folder parent) {
		this.parent = parent;
	}

	public ObjectSystemData getPredecessor() {
		return predecessor;
	}

	public void setPredecessor(ObjectSystemData previous) {
		this.predecessor = previous;
	}
	
	/**
	 * 
	 * @return the first version of this object.
	 */
	public ObjectSystemData getRoot() {
		return root;
	}

	public void setRoot(ObjectSystemData root) {
		this.root = root;
	}

	public String getName() {
		return name;
	}

	public ObjectType getType() {
		return type;
	}

	public void setType(ObjectType type) {
		this.type = type;
	}

	public User getCreator() {
		return creator;
	}

	public void setCreator(User user) {
		this.creator = user;
	}

	public Date getCreated() {
		return created;
	}

	public void setCreated(Date created) {
		this.created = created;
	}

	public Date getIndexed() {
		return indexed;
	}

	public void setIndexed(Date indexed) {
		this.indexed = indexed;
	}

	public Date getModified() {
		return modified;
	}

	public void setModified(Date modified) {
		this.modified = modified;
	}

	public Boolean getLatestBranches() {
		return latestBranches;
	}

	public void setLatestBranches(Boolean latestBranches) {
		this.latestBranches = latestBranches;
	}

	public Boolean getLatestHead() {
		return latestHead;
	}

	public void setLatestHead(Boolean latestHead) {
		this.latestHead = latestHead;
	}

	public User getModifier() {
		return modifier;
	}

	public void setModifier(User modifier) {
		this.modifier = modifier;
	}

	public User getOwner() {
		return owner;
	}

	public void setOwner(User owner) {
		this.owner = owner;
	}

	public String getProcstate() {
		return procstate;
	}

	public void setProcstate(String procstate) {
		this.procstate = procstate;
	}

	public String getVersion() {
		return version;
	}

	public void setVersion(String version) {
		this.version = version;
	}

	public User getLocked_by() {
		return locked_by;
	}

	public void setLocked_by(User locked_by) {
		this.locked_by = locked_by;
	}
	
	/**
	 * createClone: create a copy with created and modified set to current date, <br>
	 * and without an Id (which should be set by the persistence layer when inserting into  the db).
	 */
	public ObjectSystemData createClone(){
		ObjectSystemData twin = new ObjectSystemData();
		
		twin.setAcl(getAcl());
		twin.setAppName(getAppName());
		twin.setContentPath(getContentPath());
		twin.setContentSize(getContentSize());
		twin.setCreated(Calendar.getInstance().getTime());
		twin.setCreator(getCreator());
		twin.setFormat(getFormat());
		//		twin.setIndexed(null); // must be "now()", or IndexServer will do bad things.
		twin.setLanguage(getLanguage());
		twin.setLatestBranches(getLatestBranches());
		twin.setLatestHead(getLatestHead());
		twin.setLocked_by(getLocked_by());
		twin.setMetadata(getMetadata());
		twin.setModified(Calendar.getInstance().getTime());
		twin.setModifier(getModifier());
		twin.setName(getName());
		twin.setParent(getParent());
		twin.setPredecessor(getPredecessor());
		twin.setProcstate(getProcstate());
		twin.setRoot(getRoot());
		twin.setType(getType());
		twin.setVersion(getVersion());
		return twin;
	}
	
	/**
	 * @return the language
	 */
	public Language getLanguage() {
		return language;
	}

	/**
	 * @param language the language to set
	 */
	public void setLanguage(Language language) {
		this.language = language;
	}

	/**
	 * Set the objects procstate status to _done or _todo.
	 */
	public void setFinished(boolean done){
		log.debug("setting procstate to finished (_done).");
		if (done) {
			setProcstate("_done");
		} else {
			setProcstate("_todo"); 
			/*
			 * note that setReadyForValidation also may set the state to _todo.
			 * This is okay, since an unfinished task is always todo, 
			 * (unless ready for validation...)
			 */ 
		}		
	}
	
	public void updateAccess(User user)  {
		setModifier(user);
		setModified(Calendar.getInstance().getTime());
	}

	@Transient
	public boolean isFinished(){
		return getProcstate().equals("_done");
	}
	
	@Transient
	public void setReadyForValidation(boolean ready){
		if (ready) {
			setProcstate("_validate_task");
		}
		else{
			setProcstate("_todo");
		}
	}
	
	@Transient
	public boolean isReadyForValidation(){
		return getProcstate().equals("_validate_task");
	}
	
	public Document toXML() {
		Document doc = DocumentHelper.createDocument();
		doc.add(convert2domElement());
		
		return doc;
	}
	
	/**
	 * Delete the file content of this Data Object.
	 */
	public void deleteContent(String repository){
		Conf conf = ConfThreadLocal.getConf();
		String contentPath = getContentPath();
		if (contentPath != null && contentPath.length() > 0) {
			File contentFile = new File(conf.getDataRoot() + File.separator + 
					repository + File.separator + contentPath);
			log.debug("deleteContent: "+contentFile.getAbsolutePath());
			if(contentFile.exists()){
				log.debug("content exists, trying to delete");
				Boolean fileDeleted = contentFile.delete();
				if(! fileDeleted){
					throw new CinnamonException("error.delete.failed");
				}
			}
			else{
				log.warn("content file " + contentFile.getAbsolutePath() + "does not exist.");
			}
			setContentSize(null);
			setContentPath(null);
			setFormat(null);
		}
	}
	
	public void setContentPathAndFormat(String contentPath, String formatName, String repositoryName){
		EntityManager em = HibernateSession.getLocalEntityManager();
		FormatDAO formatDAO = daoFactory.getFormatDAO(em);
		Format newFormat = formatDAO.findByName(formatName);	
		setFormat(newFormat);
		setContentPathAndFormat(contentPath, newFormat, repositoryName);
	}
	
	public void setContentPathAndFormat(String contentPath, Format newFormat, String repositoryName){
		setContentPath(contentPath, repositoryName); // side effect: will set contentSize, if valid. 
		setFormat(newFormat);
	}
	
	/**
	 * Turn a collection of data objects into an XML document. Any exceptions encountered during 
	 * serialization are turned into error-Elements which contain the exception's message.
	 * @param results
	 * @return Document
	 */
	static public Document generateQueryObjectResultDocument(Collection<ObjectSystemData> results){
		return generateQueryObjectResultDocument(results, false);
	}
	
	/**
	 * Turn a collection of data objects into an XML document. Any exceptions encountered during 
	 * serialization are turned into error-Elements which contain the exception's message.
	 * @param results
	 * @return Document
	 */
	static public Document generateQueryObjectResultDocument(Collection<ObjectSystemData> results,
			Boolean withMetadata){
		Document doc = DocumentHelper.createDocument();
		Element root = doc.addElement("objects");
		Logger log = LoggerFactory.getLogger(ObjectSystemData.class);
		
		for (ObjectSystemData osd : results) {
    		Long id= osd.getId();
    		log.debug("working on object: "+id);
    		Element data;
    		try {
    			data = osd.convert2domElement();
    			if(withMetadata){
    				data.add(ParamParser.parseXml(osd.getMetadata(), null));
    			}
    			root.add(data);
    		}
    		catch(CinnamonException ex) {
    			/*
    			 * Note: any exceptions encountered here are probably serious bugs,
    			 * which could be caused by corrupted data or faulty serialization
    			 * routines.
    			 * So, let's report them as errors instead of debug messages.
    			 */
    			log.error("Error serializing object: " + id + " - " + ex.getMessage());
    			Element error = DocumentHelper.createElement("error").addText(ex.getLocalizedMessage());
    			error.addElement("id").addText(id.toString());
    			root.add(error);
    		}
    	}
		return doc;
	}
	
	public void toXmlElement(Element root){
		root.add(convert2domElement());
	}
	
	public Element convert2domElement(){
		Element data = DocumentHelper.createElement("object");
		data.addElement("id").addText(String.valueOf(getId()));
		data.addElement("name").addText(getName());	
		data.addElement("version").addText(getVersion());
		data.addElement("created").addText(ParamParser.dateToIsoString(getCreated()));
		data.addElement("modified").addText( ParamParser.dateToIsoString(getModified()));
		data.addElement("procstate").addText(getProcstate());
		data.addElement("aclId").addText(String.valueOf(getAcl().getId()));

		data.addElement("appName").addText(getAppName());
		data.addElement("latestHead").addText(getLatestHead().toString());
		data.addElement("latestBranches").addText(getLatestBranches().toString());

		log.debug("UserAsElementSection");
		data.add(User.asElement("lockedBy", getLocked_by()));
		data.add(User.asElement("owner", getOwner()));
		data.add(User.asElement("creator", getCreator()));
		data.add(User.asElement("modifier", getModifier()));
//		Element ue = User.asElement("lockedBy", getLocked_by());
//		data.add(ue);
//		
//		ue = User.asElement("owner", getOwner());
//		log.debug("owner: "+ue.asXML());
//		log.debug(ue.toString());
//		data.add(ue);
//		
//		ue = User.asElement("creator", getCreator());
//		log.debug("creator: "+ue.asXML());
//		log.debug(ue.toString());
//		data.add(ue);
//		
//		ue = User.asElement("modifier", getModifier());
//		log.debug("modifier: "+ue.asXML());
//		log.debug(ue.toString());
//		data.add(ue);
		
		log.debug("FormatAsElement");
		data.add(Format.asElement("format", getFormat()));
		log.debug("ObjectTypeAsElement");
		data.add(ObjectType.asElement("objectType", getType()));
		
		log.debug("nullChecks");
		if(getContentSize() != null){
			data.addElement("contentsize").addText(String.valueOf(getContentSize()));
		}
		else{
			data.addElement("contentsize");
		}
		
		if(getParent() != null){
			data.addElement("parentId").addText(String.valueOf(getParent().getId()));
		}
		else{
			data.addElement("parentId");
		}
		
		if(getPredecessor() != null){
			data.addElement("predecessorId").addText(String.valueOf(getPredecessor().getId()));
		}
		else{
			data.addElement("predecessorId");
		}
		
		if(getRoot() != null){
			data.addElement("rootId").addText(String.valueOf(getRoot().getId()));
		}
		else{
			data.addElement("rootId"); // TODO: prevent rootId==null on the db level.
		}
		
		log.debug("languageSection");
		Element lang = data.addElement("language"); 
		lang.addElement("id").addText(String.valueOf(getLanguage().getId()));
		lang.addElement("isoCode").addText(getLanguage().getIsoCode());

		return data;
	}

	/*
	 * Note: this tries to index all items with content-type xml.
	 * An alternative would be to just the content file over to the Indexer.
	 * (non-Javadoc)
	 * @see server.index.Indexable#getContent()
	 */
	@Override
	public String getContent(String repository) {
	
		if(contentSize == null){
			return "<content/>";
		}
		String fileContent;
		try{
			fileContent =  utils.ContentReader.readFileAsString(
				getFullContentPath(repository)
			);
		}
		catch (Exception e) {
			throw new CinnamonException(e);
		}
		return fileContent;
	}
	
	@Override
	public String getSystemMetadata() {
		log.debug("getsystemMeta");
		Document doc = DocumentHelper.createDocument();
		Element root = doc.addElement("sysMeta");
		String className = getClass().getName();
		root.addAttribute("javaClass", className);
		root.addAttribute("hibernateId", String.valueOf(getId()));
		root.addAttribute("id", className + "@" + getId()); // for a given repository, it's unique.
		log.debug("convert2domElement");
		root.add(convert2domElement());
		return doc.asXML();
	}

	/**
	 * 
	 * @param repositoryName
	 * @return the complete path to this OSD's content - or null, if no content exists.
	 */
	public String getFullContentPath(String repositoryName){
		Conf conf = ConfThreadLocal.getConf();
		if(contentPath == null){
			return null;
		}
		String fullPath = conf.getDataRoot() + File.separator +
		repositoryName + File.separator + getContentPath();
		return fullPath;
	}

	/**
	 * @return the indexStatus
	 */
	public Boolean getIndexOk() {
		return indexOk;
	}

	/**
	 * Set the index status, true means indexing was completed successfully, false means 
	 * indexing has failed. If indexOk is null, this object has not been indexed yet (or
	 * it is scheduled to be indexed as soon as possible, provided the IndexServer is running,
	 * which can be configured by setting {@code <startIndexServer>true</startIndexServer>}
	 * in the cinnamon_config.xml).
	 * @param indexOk
	 */
	public void setIndexOk(Boolean indexOk) {
		this.indexOk = indexOk;
	}

	public void copyContent(String repositoryName, ObjectSystemData copy) throws IOException{
		String conPath = getContentPath();
	    if(conPath != null && conPath.length()>0) {
	    	String fullContentPath = getFullContentPath(repositoryName);
	  	    
	  	    log.debug("ContentPath: " + fullContentPath +
	  	    		" and Size is: "+ getContentSize() );
	    	String targetPath= ContentStore.copyToContentStore(fullContentPath, repositoryName);
	    	log.debug("targetPath = "+targetPath);
	    	if(targetPath.length()>0){
	    		copy.setContentPath(targetPath, repositoryName);
	    	}
	    }	
	}
	
	@SuppressWarnings("unchecked")
	private String createNewVersionLabel(){
		
		if(getPredecessor() == null){
			return "1";
		}		

		String predecessorVersion= getPredecessor().getVersion(); 
		String[] branches=predecessorVersion.split("\\.");
		String lastSegment=branches[branches.length-1];
		String[] lastBranch=lastSegment.split("-");
		
		String lastDescendantVersion;
		try {
			log.debug("query for predecessor "+getPredecessor().getId());
			EntityManager em = HibernateSession.getLocalEntityManager();
			Query q = em.createNamedQuery("findOsdsByPredecessorOrderByIdDesc");                          
			q.setParameter("predecessor", getPredecessor());
			List<ObjectSystemData> versions = q.getResultList();
			ObjectSystemData lastDescendant;
			if(versions.size() == 0){
				throw new NoResultException();
			}
			else{
				lastDescendant = versions.get(0);
			}
			lastDescendantVersion = lastDescendant.getVersion();
		} catch (NoResultException e) {		
			// no object with same predecessor
			log.debug("no result for last-descendant-query");
			String buffer=lastBranch.length==2?lastBranch[1]:lastBranch[0];
			String stem=predecessorVersion.substring(0,predecessorVersion.length()-buffer.length());
			buffer=String.valueOf(Integer.parseInt(buffer)+1);
			return stem+buffer;
		}
		log.debug("lastDescendant: "+lastDescendantVersion);
		String[] lastDescBranches=lastDescendantVersion.split("\\.");
		if(branches.length==lastDescBranches.length) {
			// last descendant is the only one so far: create first branch
			return predecessorVersion + ".1-1";
		}
		String buffer=lastDescBranches[lastDescBranches.length-1].split("-")[0];
		buffer=String.valueOf(Integer.parseInt(buffer)+1);
		return predecessorVersion + "." + buffer + "-1";
	}


    public LifeCycleState getState() {
        return state;
    }

    public void setState(LifeCycleState state) {
        this.state = state;
    }

    @Override
	/*
	 * Implements Comparable interface based on id.
	 * This is used to generate search results with a consistent ordering (where the
	 * results are not ordered by any other parameter).
	 */
	public int compareTo(XmlConvertable o) {
		if(getId() > o.getId()){
			return 1;
		}
		else if(getId() < o.getId()){
			return -1;
		}
		return 0;
	}
	
}

