package server.index;

import server.dao.DAOFactory;
import server.dao.IndexGroupDAO;
import server.dao.IndexItemDAO;
import server.dao.IndexTypeDAO;
import server.global.Constants;
import server.index.IndexType.DataType;
import server.index.indexer.*;
import server.index.valueAssistance.DefaultProvider;

import javax.persistence.EntityManager;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * The CinnamonIndexInitializer is responsible for the creation of 
 * IndexItems and IndexTypes which index system
 * metadata about folders and objects.
 * Cinnamon can function without this class and the default objects it creates,
 * but you will probably need those anyway.
 */
public class CinnamonIndexInitializer {
	
	private EntityManager em;
	private static DAOFactory daoFactory = DAOFactory.instance(DAOFactory.HIBERNATE);
	private IndexTypeDAO itDao;
	private IndexGroupDAO igDao;
	private IndexGroup iGroup;
	
	public CinnamonIndexInitializer(EntityManager em){
		this.em = em;
		itDao = daoFactory.getIndexTypeDAO(em);
		igDao = daoFactory.getIndexGroupDAO(em);
	}
	
	/**
	 * Create index entries for folders and 
	 */
	public void createFolderSysMetaItems(){
		IndexType type = itDao.findByName("xpath.string_indexer");
		List<IndexItem> items = new ArrayList<IndexItem>();
		items.add(new IndexItem("index.folder.owner", "/sysMeta/folder/owner/id", "true()", "owner", type,
				false, "<vaParams/>", true, iGroup, false, false, true));
		items.add(new IndexItem("index.folder.name", "/sysMeta/folder/name", "true()", "name", type,
				false, "<vaParams/>", true, iGroup, false, false, true));
		items.add(new IndexItem("index.folder.type", "/sysMeta/folder/typeId", "true()", "folderType", type,
				false, "<vaParams/>", true, iGroup, false, false, true));
		
		type = itDao.findByName("xpath.integer_indexer");
		items.add(new IndexItem("index.folder.parentId", "/sysMeta/folder/parentId",
				"true()", "parent_id", type,
				false, "<vaParams/>", true, iGroup, false, false, true));
		
		type = itDao.findByName("xpath.parent_folder_path_indexer");
		items.add(new IndexItem("index.folder.path", "/sysMeta/folder/parentId", 
				"true()", "folderpath", type,
				false, "<vaParams/>", true, iGroup, false, false, true));
		persistItems(items);

	}
	
	public void createOSDSysMetaItems(){		
		IndexType type = itDao.findByName("xpath.string_indexer");
		List<IndexItem> items = new ArrayList<IndexItem>();
		// String
		items.add(new IndexItem("index.procstate", "/sysMeta/object/procstate", 
				"true()", "procstate", type,
				false, "<vaParams/>", true, iGroup, false, false, true));
		items.add(new IndexItem("index.appname", "/sysMeta/object/appname", 
				"true()", "appname", type,
				false, "<vaParams/>", true, iGroup, false, false, true));
		items.add(new IndexItem("index.version", "/sysMeta/object/version", 
				"true()", "version", type, 
				false, "<vaParams/>", true, iGroup, false, false, true));
		items.add(new IndexItem("index.name", "/sysMeta/object/name", 
				"true()", "name", type, 
				false, "<vaParams/>", true, iGroup, false, false, true));
		
		// Integer
		type = itDao.findByName("xpath.integer_indexer");
		items.add(new IndexItem("index.root", "/sysMeta/object/rootId","true()", 
				"root", type, false, "<vaParams/>", true, iGroup, false, false, true));
		items.add(new IndexItem("index.parentId", "/sysMeta/object/parentId", "true()", 
				"parent_id", type, false, "<vaParams/>", true, iGroup, false, false, true));
		items.add(new IndexItem("index.contentsize", "/sysMeta/object/contentsize",
				"true()", "contentsize", type, false, "<vaParams/>", true, iGroup, false, false, true));
		items.add(new IndexItem("index.lockedby", "/sysMeta/object/lockedBy/id", 
				"true()", "lockedby", type, false, "<vaParams/>", true, iGroup, false, false, true));
		items.add(new IndexItem("index.creator", "/sysMeta/object/creator/id", 
				"true()", "creator", type, false, "<vaParams/>", true, iGroup, false, false, true));
		items.add(new IndexItem("index.modifier", "/sysMeta/object/modifier/id", 
				"true()", "modifier", type, false, "<vaParams/>", true, iGroup, false, false, true));
		items.add(new IndexItem("index.owner", "/sysMeta/object/owner/id", 
				"true()", "owner", type, false, "<vaParams/>", true, iGroup, false, false, true));	
		items.add(new IndexItem("index.format", "/sysMeta/object/format/id", 
				"true()", "format", type, false, "<vaParams/>", true, iGroup, false, false, true));
		items.add(new IndexItem("index.objecttype", "/sysMeta/object/objectType/id", 
				"true()", "objecttype", type, false, "<vaParams/>", true, iGroup, false, false, true));
		items.add(new IndexItem("index.language", "/sysMeta/object/language/id",
				"true()", "language", type, false, "<vaParams/>", true, iGroup, false, false, true));
		items.add(new IndexItem("index.acl", "/sysMeta/object/aclId", 
				"true()", "acl", type, false, "<vaParams/>", true, iGroup, false, false, true));
		
		// Date
		type = itDao.findByName("xpath.date_indexer");
		items.add(new IndexItem("index.created.date", "/sysMeta/object/created", 
				"true()", "created.date", type, false, "<vaParams/>", true, iGroup, false, false, true));
		items.add(new IndexItem("index.modified.date", "/sysMeta/object/modified", 
				"true()", "modified.date", type, false, "<vaParams/>", true, iGroup, false, false, true));

		// Time
		type = itDao.findByName("xpath.time_indexer");
		items.add(new IndexItem("index.created.time", "/sysMeta/object/created", 
				"true()", "created.time", type, false, "<vaParams/>", true, iGroup, false, false, true));
		items.add(new IndexItem("index.modified.time", "/sysMeta/object/modified",
				"true()", "modified.time", type, false, "<vaParams/>", true, iGroup, false, false, true));
		
		// Boolean
		type = itDao.findByName("xpath.boolean_indexer");
		items.add(new IndexItem("index.latesthead", "/sysMeta/object/latestHead",
				"true()", "latesthead", type, false, "<vaParams/>", true, iGroup, false, false, true));
		items.add(new IndexItem("index.latestbranch", "/sysMeta/object/latestBranches",
				"true()", "latestbranch", type, false, "<vaParams/>", true, iGroup, false, false, true));
		
		// ParentFolderPath
		type = itDao.findByName("xpath.parent_folder_path_indexer");
		items.add(new IndexItem("index.path", "/sysMeta/object/parentId", 
				"true()", "folderpath", type, false, "<vaParams/>", true, iGroup, false, false, true));
		
		persistItems(items);	
	}
	
	public void initializeIndexTypes(){
		List<IndexType> itList = new ArrayList<IndexType>();
		
		itList.add(new IndexType("xpath.boolean_indexer", 
				BooleanXPathIndexer.class, DefaultProvider.class, 
				DataType.BOOLEAN));
		
		itList.add(new IndexType("xpath.date_indexer",
				DateXPathIndexer.class, DefaultProvider.class, DataType.DATE_TIME));
	
		itList.add(new IndexType("xpath.integer_indexer",
				IntegerXPathIndexer.class, DefaultProvider.class, DataType.INTEGER));
		
		itList.add(new IndexType("xpath.date_time_indexer",
				DateTimeIndexer.class, DefaultProvider.class, DataType.INTEGER));
		
		itList.add(new IndexType("xpath.string_indexer", 
				DefaultIndexer.class, DefaultProvider.class, DataType.TEXT));
		
		itList.add(new IndexType("xpath.decimal_indexer", 
				DecimalXPathIndexer.class, DefaultProvider.class, DataType.DECIMAL));
		
		itList.add(new IndexType("xpath.time_indexer", 
				TimeXPathIndexer.class, DefaultProvider.class, DataType.TIME));
		
		itList.add(new IndexType("xpath.reverse_string_indexer",
				ReverseStringIndexer.class, DefaultProvider.class, DataType.STRING));
		
		itList.add(new IndexType("xpath.reverse_complete_string_indexer",
				ReverseCompleteStringIndexer.class, DefaultProvider.class, DataType.STRING));
		
		itList.add(new IndexType("xpath.parent_folder_path_indexer",
				ParentFolderPathIndexer.class, DefaultProvider.class, DataType.STRING));
		
		for(IndexType type : itList){
			itDao.makePersistent(type);
		}
	}
	
	public IndexGroup createDefaultIndexGroup(){
		iGroup = new IndexGroup(Constants.INDEX_GROUP_DEFAULT);
		igDao.makePersistent(iGroup);
		return iGroup;
	}
	
	void persistItems(Collection<IndexItem> items){
		IndexItemDAO itemDao = daoFactory.getIndexItemDAO(em);
		for(IndexItem item : items){
			itemDao.makePersistent(item);
		}
	}
	
}
