package server.index;

import org.apache.lucene.document.Document;
import org.dom4j.Element;
import org.hibernate.annotations.Type;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import server.global.Constants;
import utils.ParamParser;

import javax.persistence.*;
import java.io.Serializable;

@Entity
@Table(name="index_items",
		uniqueConstraints = {@UniqueConstraint(columnNames={"name"})}		
)
public class IndexItem implements Serializable{

	/**
	 * 
	 */
	private static final long	serialVersionUID	= 1L;
	@Transient
	transient Logger log = LoggerFactory.getLogger(this.getClass());
	
	@Id @GeneratedValue
	@Column(name = "id")
	long id;
	
	@Version
	@Column(name="obj_version",
			nullable = false)
	@SuppressWarnings("unused")
	private Long obj_version;
	
	@Column(name = "name",
			length = Constants.NAME_LENGTH,
			nullable = false)
//	@Type(type="text")
	String name;
	
	@Column(name = "search_string",
			length = Constants.XPATH_LENGTH,
			nullable = false)
	@Type(type="text")
	String searchString = "";
	
	/**
	 * searchCondition is an XPath expression that evaluates to true
	 * if the IndexItem should try to index the given document.<br/>
	 * The XPath will be applied successively to the metadata, system 
	 * metadata and content (depending on forContent, forMetadata and
	 * forSysMetadata)
	 * and if any of those returns true, the item
	 * will be indexed by using the search_string.
	 */
	@Column(name = "search_condition",
			length = Constants.XPATH_LENGTH,
			nullable = false)
	@Type(type="text")
	String searchCondition = "true()";
	
	@Column(name = "fieldname",
			length = 255,
			nullable = false)
//	@Type(type="text")
	String fieldname = "content";
	
	@ManyToOne
	@JoinColumn(name = "index_type_id",
				nullable = false)
	IndexType indexType;

	@Column(name = "system_index",
			nullable = false)
	Boolean	systemic = false;
	
	@ManyToOne
	@JoinColumn(name = "index_group_id",
				nullable = false)
	IndexGroup indexGroup;
	
	@Column(name = "for_content",
			nullable = false)
	Boolean forContent = false;
	
	@Column(name = "for_metadata",
			nullable = false)
	Boolean forMetadata = false;
	
	@Column(name = "for_sysmeta",
			nullable = false)
	Boolean forSysMeta = false;
	
	/**
	 * If an indexItem can generate multiple distinct results, you should set this
	 * flag to true. For example, an XML structure containing a list of "filename"
	 * elements which is queries by an xpath statement of //filename will generate a
	 * list of nodes, which should all be indexed. On the other hand, a UUID search
	 * field will only contain 1 result. 
	 */
	@Column(name = "multiple_results",
			nullable = false)
	Boolean multipleResults = false;
	
	/**
	 * Each IndexType is connected to a specific ValueAssistanceProvider, which is called
	 * upon to provide a list of possible values for a search field on the client.<br/>
	 * For example, if you index a field "birthday", the acceptable values represent
	 * to a certain date in the past - the client should not submit search requests for
	 * values like "birthday=John" but rather tell the user to correct the input. 
	 */
	@Column(name = "va_params",
			length = Constants.XML_PARAMS,
			nullable = false)
	@Type(type="text")		
	String vaProviderParams = "<vaParams/>";
	
	public IndexItem(){
		
	}
	
	public IndexItem(String name, String xpath, String searchCondition, String fieldname, IndexType indexType,
			Boolean multipleResults, String vaParams, Boolean systemic, IndexGroup indexGroup,
			Boolean forContent, Boolean forMetadata, Boolean forSysMeta){
		this.name = name;
		this.searchString = xpath;
		this.searchCondition = searchCondition;
		this.fieldname = fieldname;
		this.indexType = indexType;
		this.multipleResults = multipleResults;
		this.vaProviderParams = vaParams;
		this.systemic = systemic;
		this.indexGroup = indexGroup;
		this.forContent = forContent;
		this.forMetadata = forMetadata;
		this.forSysMeta = forSysMeta;
	}
	
	/*
	 * revision 1040
	 * A note on class design: this class tries to adhere to the principle of strict
	 * encapsulation, so access to fields inside the class is without
	 * getters/setters at the moment. It's an experiment, and you may refactor
	 * the class if you need to. An alternative would be to only use private getter/setters,
	 * but that will only be useful if this class gets validation logic on get/set
	 * or something like that.
	 */
	public void toXmlElement(Element root){
		Element item = root.addElement("indexItem");
		item.addElement("id").addText(String.valueOf(id));
		item.addElement("name").addText(name);
		item.addElement("searchString").addText(searchString);
		item.addElement("fieldname").addText(fieldname);
		indexType.toXmlElement(item);
		item.addElement("multipleResults").addText(multipleResults.toString());		
		item.add(ParamParser.parseXml(vaProviderParams, "error.parse.va_params"));
		item.addElement("systemic").addText(systemic.toString());
		item.addElement("forMetadata").addText(forMetadata.toString());
		item.addElement("forContent").addText(forContent.toString());
		item.addElement("forSysMeta").addText(forSysMeta.toString());
		indexGroup.toXmlElement(item);
	}
	
	public void indexObject(String content, String metadata, String systemMetadata, Document doc){
		String[] params = {content, metadata, systemMetadata};
		if(! checkCondition(params)){
			log.debug("checkCondition returned false");
			return;
		}
        log.debug("checkCondition returned true");
		if(forContent){
			indexType.indexContent(content, doc, fieldname, searchString, multipleResults);
		}
		if(forSysMeta){
			indexType.indexSysMeta(systemMetadata, doc, fieldname, searchString, multipleResults);
		}
		if(forMetadata){
			indexType.indexMetadata(metadata, doc, fieldname, searchString, multipleResults);
		}
	}
	
	/**
	 * Check if one of the given parameter Strings has a positive result for the
	 * xpath expression in searchCondition.
	 * @param params an array of strings which contain XML documents.
	 * @return true if one of the strings resulted in a positive match for searchCondition.
	 */
	public Boolean checkCondition(String[] params){
		Boolean result = false;
		for(String xml : params){
			try{
				org.dom4j.Document indexObject = ParamParser.parseXmlToDocument(xml, "error.parse.indexable_object");
				log.debug("checkCondition "+searchCondition+": "+indexObject.valueOf(searchCondition)) ;
				if(indexObject.valueOf(searchCondition).equals("true")){
					result = true;
					break;
				}				
			}
			catch (Exception e) {
				log.debug("checkCondition: ",e.getMessage());
			}
		}
		return result;
	}
	
	public Long getId(){
		return id;
	}
}
