package server.index;

import org.apache.lucene.document.Document;
import org.dom4j.Element;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import server.global.Constants;
import server.i18n.LocalMessage;

import javax.persistence.*;
import java.io.Serializable;

/**
 * Distinguish multiple IndexItems by their type.
 *
 */

@Entity
@Table(name="index_types",
		uniqueConstraints = {@UniqueConstraint(columnNames={"name"})}
)
public class IndexType implements Serializable {

	/**
	 * 
	 */
	private static final long	serialVersionUID	= 1L;
	@Transient
	transient Logger log = LoggerFactory.getLogger(this.getClass());

	public enum DataType{
		STRING, BOOLEAN, DATE_TIME, INTEGER, DECIMAL, TIME, TEXT
	}
	
	@Id @GeneratedValue
	@Column(name = "id")
	long id;
	
	@Version
	@Column(name = "obj_version",
			nullable = false)
	@SuppressWarnings("unused")
	private Long obj_version;
	
	@Column(name = "name",
			length = Constants.NAME_LENGTH,
			nullable = false)
//	@Type(type="text")
	String name;
	
	@Column(name = "indexer_class",
			nullable = false)
	Class<? extends Indexer> indexerClass;
	
	@Column(name = "va_provider_class",
			nullable = false)
	Class<? extends ValueAssistanceProvider> vaProviderClass;
	
	@Column(name = "data_type",
			nullable = false)
	@Enumerated(EnumType.STRING)
	DataType dataType;
	
	public IndexType(){
		
	}
	
	public IndexType(String name, Class<? extends Indexer> indexerClass,
			Class<? extends ValueAssistanceProvider> vaProviderClass,
			DataType dataType){
		this.name = name;
		this.indexerClass = indexerClass;
		this.dataType = dataType;
		this.vaProviderClass = vaProviderClass;
	}
	
	public void toXmlElement(Element root){
		Element type = root.addElement("indexType");
		type.addElement("id").addText(String.valueOf(id));
		type.addElement("name").addText( LocalMessage.loc(name));
		type.addElement("indexerClass").addText(indexerClass.getName());
		type.addElement("dataType").addText(dataType.toString());
		type.addElement("vaProviderClass").addText(vaProviderClass.getName());
	}

	public Indexer getIndexer(){
		Indexer indexer;
		try{
//            log.debug("instantiating: "+indexerClass.getName());
			indexer = indexerClass.newInstance();
		}
		catch (IllegalArgumentException e) {
			throw new RuntimeException(e);
		} catch (InstantiationException e) {
			throw new RuntimeException(e);
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e);
		}
		return indexer;
	}


	public void indexContent(String content, Document doc, String fieldname, String searchString, Boolean multipleResults){
		getIndexer().indexObject(content, doc, fieldname, searchString, multipleResults);
	}
	
	public void indexSysMeta(String sysMeta, Document doc, String fieldname, String searchString, Boolean multipleResults){
		getIndexer().indexObject(sysMeta, doc, fieldname, searchString, multipleResults);
	}
	
	public void indexMetadata(String metadata, Document doc, String fieldname, String searchString, Boolean multipleResults){
		getIndexer().indexObject(metadata, doc, fieldname, searchString, multipleResults);
	}
}
