package server.index;

import java.util.Date;

public interface Indexable {

	/**
	 * An Indexable object should have a field with XML metadata, which is represented as a String.
	 * For example, an image object may have it's width and height recorded as metadata.<br>
	 * A PDF manual may have the type of engines it is applicable for stored in the metadata field.   
	 * @return XML metadata or an empty meta element: {@code <meta/>} 
	 */
	String getMetadata();
	
	/**
	 * <p>If an object contains indexable strings, it should return them according to a fixed XML
	 * Schema. For example, an XML Document should be returned verbatim. A folder may represent its
	 * content as an XML document. An image document may either return an empty XML document or a automatically
	 * generated OCR representation.
	 * </p>
	 * <p>Note: you should not return the same data as Metadata and as content unless it is really 
	 * necessary.</p> 
	 * @return an XML representation of the object's content, as appropriate, or an empty content
	 * element: {@code <content/>};
	 */
	String getContent(String repository);
	
	/**
	 * <p>Return an XML representation of the object's system metadata. This is whatever the system itself
	 * knows about the object. For example, who created the object, and when.</p>
	 * <p>The root element of the XML is "sysMeta" and it must have the following two attributes set:</p>
	 * <ol>
	 * <li>javaClass</li>
	 * <li>hibernateId</li>
	 * </li>
	 * <p>The LuceneBridge will generate a unique Id from this ($className+$hibernateId).</p> 
	 * @return an XML representation of the object's system metadata - 
	 * or an empty {@code <sysMeta/>} element with attributes javaClass and hibernateId.
	 */
	String getSystemMetadata();
	
	/**
	 * <p>If indexing was successful, set indexOk to true, otherwise to false.</p>
	 * <p>If this field is null, the item in question has not been indexed yet.<p>
	 * @return true if indexing was successful, false otherwise.
	 */
	Boolean getIndexOk();
	void setIndexOk(Boolean indexOk);
	void setIndexed(Date date);
}
