package server.index;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.Field.Index;
import org.apache.lucene.document.Field.Store;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.IndexWriter.MaxFieldLength;
import org.apache.lucene.index.Term;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.SimpleFSDirectory;
import org.apache.lucene.util.Version;
import org.apache.lucene.xmlparser.CoreParser;
import org.apache.lucene.xmlparser.ParserException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import server.dao.DAOFactory;
import server.dao.IndexItemDAO;
import server.global.ConfThreadLocal;
import server.index.queryBuilder.RegexQueryBuilder;
import server.index.queryBuilder.WildcardQueryBuilder;
import utils.ParamParser;

import javax.persistence.EntityManager;
import java.io.*;
import java.util.Date;
import java.util.List;
import java.util.Properties;

/**
 * LuceneBridge provides the following functionality:<br>
 * <ul>
 * <li>Send your indexable items over the bridge into Lucene's index.</li>
 * <li>Fetch those items back from the index via a search.</li>
 * <li>Throw old items over the bridge when they have been deleted in the repository 
 * and thereby purge them from the index.</li>
 * </ul>
 * <p>In less fanciful prose: you can use this class to add items to the index, search for 
 * them and remove them from the index. To update an item, you have to remove it from
 * the index and add it again (this is the recommended Lucene way).</p>
 * <p>Note: At the moment this class uses a rather expensive approach as it will open the
 * index for reading and writing each time an object is changed or searched. That's quite a
 * lot I/O. The index server will reduce this probably by re-using the IndexReader/IndexSearcher
 * objects.</p> 
 */
public class LuceneBridge {
	
	static Properties luceneProperties = ConfThreadLocal.getConf().findProperties("lucene.properties");
	transient Logger log = LoggerFactory.getLogger(this.getClass());
	static DAOFactory daoFactory = DAOFactory.instance(DAOFactory.HIBERNATE);
	
	File indexFolder;
	Directory indexDir;
	Analyzer analyzer = new StandardAnalyzer(Version.LUCENE_30);//new CinnamonStandardAnalyzer(Version.LUCENE_CURRENT);
	String repository;
	IndexItemDAO iiDao;
	List<IndexItem> iiList;// = iiDao.list();
	
	public LuceneBridge(){}
	
	public LuceneBridge(String repository, EntityManager em){
		this.repository = repository;
		indexFolder = new File(luceneProperties.getProperty("indexDir") + 
				File.separator + repository);
		try{
			indexDir = new SimpleFSDirectory(indexFolder);
			/*
			 * Set timeout for write-locks. 
			 * TODO: timeouts should be > than a single re-index run.
			 */
			if(luceneProperties.containsKey("writeLockTimeout")){
				Long timeout = Long.parseLong(luceneProperties.getProperty("writeLockTimeout"));
				IndexWriter.setDefaultWriteLockTimeout(timeout);
			}
		}
		catch (IOException ie) {
			throw new RuntimeException(ie);
		}
		this.iiDao = daoFactory.getIndexItemDAO(em);
		this.iiList = iiDao.list();
		log.debug("# of IndexItems found: "+iiList.size());
	}

    public void addObjectToIndex(Indexable indexable){
        try{
            log.debug("create new IndexWriter");
//			synchronized(getClass()){
            Boolean create = ! IndexReader.indexExists(indexDir);
//				unlockDir(indexDir);
            IndexWriter writer = new IndexWriter(indexDir, analyzer, create, MaxFieldLength.UNLIMITED);
            try{
                Document doc = new Document();
                log.debug("store standard fields");
                storeStandardFields(indexable, doc);
                log.debug("finished store standard fields");
                String content = indexable.getContent(repository);
                log.debug("finished: getContent");
                String metadata = indexable.getMetadata();
                log.debug("store systemMetadata");
                String systemMetadata = indexable.getSystemMetadata();
                log.debug("got sysMetadata, start indexObject loop");

                for(IndexItem item : iiList){
                    /*
                              * At the moment, the OSDs and Folders do not cache
                              * their responses to getSystemMetadata or getContent.
                              * In a repository with many IndexItems, this would cause
                              * quite some strain on the server's resources.
                              */
                    try{
//							log.debug("indexObject for field '"+item.fieldname+"' with content: "+content);
                        item.indexObject(content, metadata, systemMetadata, doc);
                    }
                    catch (Exception e) {
                        log.debug("*** failed *** to execute IndexItem "+item.getId(), e);
                    }
                }
                log.debug("before addDocument");
                writer.addDocument(doc);
                log.debug("added Document");
                indexable.setIndexed(new Date());
                indexable.setIndexOk(true);
            }
            finally{
                // the writer should be closed in any case.
                try{
                    log.debug("closing writer");
                    writer.close();
                }
                finally{
                    log.debug("check IndexWriter.isLocked");
                    if(IndexWriter.isLocked(indexDir)){
                        /*
                                   * Access to the index should be synchronized and
                                   * therefore safe.
                                   */
                        log.debug("unlocking indexDir");
                        IndexWriter.unlock(indexDir);
                    }
                    else{
                        log.debug("indexDir was not locked. Ok.");
                    }
                }
            }
        }
		catch (IOException ie) {
			log.debug("IOException during indexing.",ie);
			throw new RuntimeException(ie);
		}
        catch (Exception e){
            log.debug("addObjectToIndex failed:", e);
            throw new RuntimeException(e);
        }
		log.debug("finished addObjectToIndex");
	}
	
	void unlockDir(Directory dir){
		try{
			if(IndexReader.indexExists(indexDir)){
				log.debug("indexDir "+indexDir.toString()+" does not exist.");
				return;
			}
			if(IndexWriter.isLocked(dir)){
				IndexWriter.unlock(dir);
			}
		}
		catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * Search for all documents matching the given params, which must be an
	 * Lucene XML-Query-Parser document.
	 * @param params input for XML-Query-Parser
	 * @return a ResultCollector, which contains a collection of all documents found. 
	 */
	public ResultCollector search(String params){
		log.debug("starting search");
		ResultCollector results = new ResultCollector();
		try{
			InputStream bais = new ByteArrayInputStream(params.getBytes("UTF-8"));
			CoreParser coreParser = new CoreParser("content", analyzer);
			coreParser.addQueryBuilder("WildcardQuery", new WildcardQueryBuilder());
			coreParser.addQueryBuilder("RegexQuery", new RegexQueryBuilder());
			Query query = coreParser.parse(bais);
			
			// create index if needed:
			Boolean create = ! IndexReader.indexExists(indexDir);
			if(create){
				IndexWriter writer = new IndexWriter(indexDir, analyzer, create, MaxFieldLength.UNLIMITED);
				writer.close();
			}
			
			IndexSearcher searcher = new IndexSearcher(indexDir, true);
			results.setSearcher(searcher);
			searcher.search(query, results);
			searcher.close();
		}
		catch (IOException iie) {
			throw new RuntimeException(iie);
		}
		catch (ParserException pee) {
			throw new RuntimeException(pee);
		}
		log.debug("finished search; results: "+results.getDocuments().size());
		return results;
	}

	/**
	 * Remove an Indexable object from the index.
	 * @param indexable
	 */
	public void removeObjectFromIndex(Indexable indexable){
		try{
			log.debug("check if index exists");
			if(!IndexReader.indexExists(indexDir)){
				log.debug("Index does not exist");
				// nothing to do.
				return;
			}
			if(indexable == null){
				log.debug("indexable is NULL");
				return;
			}
			log.debug("creating reader");
			log.debug("parsing sysmeta");
//			log.debug(indexable.getSystemMetadata());			
			org.dom4j.Document sysMeta = ParamParser.parseXmlToDocument(indexable.getSystemMetadata(),
			"error.invalid.system_metadata");
			String id = sysMeta.valueOf("/sysMeta/@id");
			log.debug("removing document with id "+id);
			log.debug("delete document");
			
			deleteDocument(new Term("uniqueId", id), 5 );
		}
		catch(FileNotFoundException f){			
			log.warn("File not found - if the index does not yet exist, "+
					"removeObjectFromIndex is expected to fail", f);
		}
		catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * Delete the documents found by the given Term from this Bridge's index.
	 *<strike> This method is synchronized on LuceneBridge.class, so you should not need
	 * to use retries > 1.</strike>
	 * @param term the search term
	 * @param retries how often to retry deleting the document. Probably useful only
	 * if other programs / servlets are also interacting with the index. 
	 */
	void deleteDocument(Term term, Integer retries) throws IOException{
//		synchronized(getClass()){
			IndexReader reader = IndexReader.open(indexDir, false);
			try{
				reader.deleteDocuments(term);
			}
			catch (Exception e) {
				log.debug("",e);
				log.debug("retry-delete document");
				deleteDocument(term, --retries);
			}
			finally{
				reader.close();
			}
//		}
	}
	
	
	/**
	 * Remove an old Indexable from the index and add it anew, so
	 * all fields are updated. The hibernateId and class of the indexable
	 * must never be changed (otherwise Lucene will not be able to find
	 * the Indexable in its index and you will be plagued by ghost objects
	 * until the next complete re-index run). 
	 * @param indexable
	 */
	public void updateObjectInIndex(Indexable indexable){
		log.debug("remove object from index");
		removeObjectFromIndex(indexable);
		log.debug("add object to index.");
		addObjectToIndex(indexable);
	}

	/**
	 * Fetch the information that every indexed object needs:
	 * a unique id and its class.
	 * @param indexable
	 * @param doc
	 */
	void storeStandardFields(Indexable indexable, Document doc){
		log.debug("start. sSF");
//		log.debug("storeStandardFields: "+indexable.getSystemMetadata());
		org.dom4j.Document sysMeta = ParamParser.parseXmlToDocument(indexable.getSystemMetadata(),
				"error.invalid.system_metadata");
		String hibernateId = sysMeta.valueOf("/sysMeta/@hibernateId");
		String className = sysMeta.valueOf("/sysMeta/@javaClass");
		String uniqueId = sysMeta.valueOf("/sysMeta/@id");
		log.debug("indexing of: "+uniqueId);
//		log.debug(String.format("adding fields: %s, %s, %s",hibernateId, className, uniqueId));
		doc.add(new Field("hibernateId", hibernateId, Store.YES, Index.NOT_ANALYZED));
		log.debug("stored hibernateId");
		doc.add(new Field("javaClass", className, Store.YES, Index.NOT_ANALYZED));
		log.debug("stored javaClass");
		doc.add(new Field("uniqueId", uniqueId, Store.YES, Index.NOT_ANALYZED));
		log.debug("stored uniqueId");
	}
	
	/**
	 * Remove all documents belonging to a given Java class from the
	 * index. <em>Warning</em>: this will remove <i>ALL</remove> documents
	 * and is intended only for clearing an index for testing purposes.  
	 * @param clazz Class of the documents you wish to remove. 
	 */
	public void removeClassFromIndex(Class<? extends Indexable> clazz){
		try{
			// create index if needed:
			if(! IndexReader.indexExists(indexDir)){
				return;
				// nothing to do - index is empty.
			}
			IndexReader reader = IndexReader.open(indexDir, false);
			reader.deleteDocuments(new Term("javaClass",clazz.getName()));
			reader.close();
		}
		catch (IOException e) {
			log.warn("IOException occured during removeClassFromIndex", e);
			throw new RuntimeException(e);
		}
	}

	String getRepository(){
		return repository;
	}

	
	/**
	 * Lucene caches the available index items for performance reasons.
	 * If you need to update the IndexItem cache without a server restart,
	 * call setIndexItemList(indexItemDao.list()).
	 * Note: you should do this only in a synchronized context or during testing,
	 * when no Lucene indexing is ongoing.
	 */
	public void setIndexItemList(List<IndexItem> items){
		iiList = items;
	}
}
