package server.trigger;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.persistence.*;
import java.io.Serializable;

/**
 * A ChangeTrigger is a class which decouples a CinnamonMethod and a Trigger-class. Every time an API-method
 * of the CinnamonServer is invoked which is marked with CinnamonMethod@trigger=true, the server checks 
 * if a trigger condition is defined and executes the Trigger-classes before and after invoking the command.
 * <br/>
 * <h2>Purpose of ChangeTriggers</h2>
 * Add transformation functions for specific object types, for example have a PDF-renderer create a new
 * PDF rendition every time an object of ObjectType "document" is changed. 
 */
@Entity
@Table(name = "change_triggers")
public class ChangeTrigger implements Serializable {

	private static final long	serialVersionUID	= 1L;

	@Transient
	transient Logger log = LoggerFactory.getLogger(this.getClass());
	
	@Id @GeneratedValue
	@Column(name = "id")
	private long 	id;
	
	@Version
	@Column(name="obj_version")
	@SuppressWarnings("unused")
	private Long obj_version = 0L;

	@ManyToOne
	@JoinColumn(name = "trigger_type_id",
				nullable = false)
	private ChangeTriggerType triggerType;
	
	@Column(name="ranking",
			nullable = false)
	private Integer ranking = 1;
	
	@Column(name = "command",
			nullable = false)
	private String command;
	
	@Column(name = "active",
			nullable = false)
	private Boolean active = false; 
	
	@Column(name = "pre_trigger",
			nullable = false)
	private Boolean preTrigger = false;
	
	@Column(name = "post_trigger",
			nullable = false)
	private Boolean postTrigger = false;
			
	public ChangeTrigger(){}
	
	public ChangeTrigger(String command, ChangeTriggerType ctt){
		triggerType = ctt;
	}
	
	public ChangeTrigger(String command, ChangeTriggerType ctt, Integer ranking,
			Boolean active, Boolean preTrigger, Boolean postTrigger){
		triggerType = ctt;
		this.ranking = ranking;
		this.command = command;
		this.active = active;
		this.preTrigger = preTrigger;
		this.postTrigger = postTrigger;
	}

	/**
	 * @return the id
	 */
	public long getId() {
		return id;
	}

	/**
	 * @param id the id to set
	 */
	@SuppressWarnings("unused")
	private void setId(long id) {
		this.id = id;
	}

	/**
	 * @return the triggerType
	 */
	public ChangeTriggerType getTriggerType() {
		return triggerType;
	}

	/**
	 * @param triggerType the triggerType to set
	 */
	public void setTriggerType(ChangeTriggerType triggerType) {
		this.triggerType = triggerType;
	}

	/**
	 * @return the ranking
	 */
	public Integer getRanking() {
		return ranking;
	}

	/**
	 * If there are several ChangeTriggers for a command, ranking allows you to sort them according
	 * to their importance. Items of ranking 1 are executed before items of ranking 2.
	 * @param ranking the ranking to set
	 */
	public void setRanking(Integer ranking) {
		this.ranking = ranking;
	}

	/**
	 * @return the active
	 */
	public Boolean getActive() {
		return active;
	}

	/**
	 * @param active the active to set
	 */
	public void setActive(Boolean active) {
		this.active = active;
	}

	/**
	 * @return the command
	 */
	public String getCommand() {
		return command;
	}

	/**
	 * @param command the command to set
	 */
	public void setCommand(String command) {
		this.command = command;
	}

	/**
	 * @return the preTrigger
	 */
	public Boolean getPreTrigger() {
		return preTrigger;
	}

	/**
	 * @param preTrigger the preTrigger to set
	 */
	public void setPreTrigger(Boolean preTrigger) {
		this.preTrigger = preTrigger;
	}

	/**
	 * @return the postTrigger
	 */
	public Boolean getPostTrigger() {
		return postTrigger;
	}

	/**
	 * @param postTrigger the postTrigger to set
	 */
	public void setPostTrigger(Boolean postTrigger) {
		this.postTrigger = postTrigger;
	}
	
}
